//
//  TitleLayer.m
//  Game01
//
//  Created by Kansuke Nishida on 11/09/18.
//  Copyright 2011年 tonosamart. All rights reserved.
//

#import "GameScene.h"
#import "SimpleAudioEngine.h"
#import "TitleLayer.h"
#import "Constants.h"

@interface TitleLayer (private)
-(void)initLayer;
-(void)tick:(ccTime)dt;
@end

@implementation TitleLayer

enum {
    SPRITE_NULL,
    SPRITE_MAP,
    SPRITE_PLAYER,
    SPRITE_TEXT,
    SPRITE_MENU
};

- (id)init{
    self = [super init];
    if (self) {
        // イベントの無効/有効
        self.isTouchEnabled = YES;
        self.isAccelerometerEnabled = NO;
        
        //レイヤーの初期化
        [self initLayer];
    }
    
    return self;
}

#pragma mark private

-(void)initLayer{
    CCSprite* sprite = [CCSprite spriteWithFile:@"free02_194.png"];
    sprite.position = ccp(240, 160);
    [self addChild:sprite z:SPRITE_PLAYER tag:SPRITE_PLAYER];
    
    //---
    CCLabelTTF* label = [CCLabelTTF labelWithString:@"" fontName:@"Marker Felt" fontSize:24];
    label.color = ccc3(0, 0, 0);
    label.position = ccp(0, 320);
    label.anchorPoint = ccp(0, 1);
    [self addChild:label z:SPRITE_TEXT tag:SPRITE_TEXT]; 
    
    //---
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"coin.caf"];
    
    //---
	// map init
	CCTMXTiledMap* tileMap = [CCTMXTiledMap tiledMapWithTMXFile:@"dungeon.tmx"];
	CCTMXLayer* moveLayer = [tileMap layerNamed:@"move"];
	moveLayer.visible = NO;
    tileMap.anchorPoint = ccp(0, 0);
    tileMap.position = ccp(0 ,0);
	[self addChild:tileMap z:SPRITE_MAP tag:SPRITE_MAP];
    
    // object
	CCTMXObjectGroup* objectGroup = [tileMap objectGroupNamed:@"object"];
    
    // player init
    NSMutableDictionary *startPoint = [objectGroup objectNamed:@"player"];        
    int sx = [[startPoint valueForKey:@"x"] intValue] + (tileMap.tileSize.width / 2);
    int sy = [[startPoint valueForKey:@"y"] intValue] + (tileMap.tileSize.height / 2);
    sprite.position = ccp(sx, sy);
    [Constants setViewpointScreen:self tileMap:tileMap position:sprite.position];
    
	// propertie
	NSDictionary* properties = [tileMap properties];
    
	// stageName
    NSString *stageName = [properties valueForKey:@"stageName"];
    if(stageName){
        label.string = stageName;
    }
    
    //---
    [self schedule:@selector(tick:)];
}

-(void)tick:(ccTime)dt{
    if([Constants isAction] == TRUE){
        return;
    };

    Stick* stick = [Stick stick];
    
    CCTMXTiledMap* tileMap = (CCTMXTiledMap*)[self getChildByTag:SPRITE_MAP];
	CCTMXLayer* moveLayer = [tileMap layerNamed:@"move"];
	CCTMXLayer* dotLayer = [tileMap layerNamed:@"dot"];
    
    CCSprite* sprite = (CCSprite*)[self getChildByTag:SPRITE_PLAYER];
    CGPoint nextPosition = ccp(sprite.position.x + (stick.x * 100 * dt), sprite.position.y + (stick.y * 100 * dt));
    CGPoint nextMapPos = [Constants mapPosFromPositon:tileMap position:nextPosition];
    
    int nextGid = [moveLayer tileGIDAt:nextMapPos];
    NSDictionary *properties = [tileMap propertiesForGID:nextGid];
    if(properties){
        NSString *collision = [properties valueForKey:@"isWall"];
        if(collision){
            if([collision compare:@"TRUE"] == NSOrderedSame){
                nextPosition = sprite.position;
            }
        }
    }
    sprite.position = nextPosition;
    [Constants setViewpointCenterFitDistance:self tileMap:tileMap position:sprite.position distance:CGSizeMake(120, 80)];
    
    CGPoint moveMapPos = [Constants mapPosFromPositon:tileMap position:nextPosition];
    int moveGid = [dotLayer tileGIDAt:moveMapPos];
    if(moveGid != 0){
        [dotLayer removeTileAt:moveMapPos];
    }
    
    if(stick.isA){
    }
    if(stick.isB){
    }
    if(stick.isC){
    }
}

#pragma mark EVENT

-(void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	for(UITouch* touch in touches){
		CGPoint touchPoint = [touch locationInView:[touch view]];
		touchPoint = [Constants convertToGL:self touchPoint:touchPoint];
		
		if(CGRectContainsPoint([Constants rectForSprite:(CCSprite*)[self getChildByTag:SPRITE_PLAYER]], touchPoint)){
            [[SimpleAudioEngine sharedEngine] playEffect:@"coin.caf"];
		}
	}
}

-(void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
}

-(void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
}

-(void)ccTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
}

@end