//
//  Constants.m
//  Game01
//
//  Created by Kansuke Nishida on 11/09/23.
//  Copyright 2011年 tonosamart. All rights reserved.
//

#import "Constants.h"

@implementation Constants

static Boolean _isAction = FALSE;

+(Boolean)isAction{
    return _isAction;
}

+(void)setAction:(Boolean)isAction{
    _isAction = isAction;
}

+(void)setActionFalse{
    _isAction = FALSE;
}

//---

+(CGRect)rectForSprite:(CCSprite *)sprite{
	float w = sprite.contentSize.width * sprite.scaleX;
	float h = sprite.contentSize.height * sprite.scaleY;
	float x = sprite.position.x - (w * sprite.anchorPoint.x);
	float y = sprite.position.y - (h * sprite.anchorPoint.y);
    return CGRectMake(x, y, w, h);
}

+(CGPoint)mapPosFromPositon:(CCTMXTiledMap*)tileMap position:(CGPoint)position {
    CGPoint pos = ccpSub(position, tileMap.position);
    int x = (int)(pos.x / tileMap.tileSize.width);
    int y = (int)(((tileMap.mapSize.height * tileMap.tileSize.height) - pos.y) / tileMap.tileSize.height);
    return ccp(x, y);
}

+(CGPoint)convertToGL:(CCLayer*)layer touchPoint:(CGPoint)touchPoint{
    CGPoint retPoint = [[CCDirector sharedDirector] convertToGL:touchPoint];
    return ccpSub(retPoint, layer.position);
}

+(void)setViewpointCenter:(CCLayer*)layer tileMap:(CCTMXTiledMap*)tileMap position:(CGPoint)position {
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    CGPoint viewPoint = ccpSub(ccp(winSize.width / 2 , winSize.height / 2), position);
	int x = (viewPoint.x * layer.scaleX);
	int y = (viewPoint.y * layer.scaleY);
    
    layer.position = ccp(x, y);
}

+(void)setViewpointCenterFit:(CCLayer*)layer tileMap:(CCTMXTiledMap*)tileMap position:(CGPoint)position {
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    CGPoint viewPoint = ccpSub(ccp(winSize.width / 2, winSize.height / 2), position);
    
    viewPoint.x = MIN(viewPoint.x, 0);
    viewPoint.y = MIN(viewPoint.y, 0);

    int mx = (((int)tileMap.mapSize.width * (int)tileMap.tileSize.width) - (int)winSize.width) * -1;
    int my = (((int)tileMap.mapSize.height * (int)tileMap.tileSize.height) - (int)winSize.height) * -1;

    viewPoint.x = MAX(viewPoint.x, mx);
    viewPoint.y = MAX(viewPoint.y, my);

	int x = (viewPoint.x * layer.scaleX);
	int y = (viewPoint.y * layer.scaleY);
    
    layer.position = ccp(x, y);
}

+(void)setViewpointCenterFitDistance:(CCLayer*)layer tileMap:(CCTMXTiledMap*)tileMap position:(CGPoint)position distance:(CGSize)distance {
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    int lpx = (int)(winSize.width / 2) - (int)(layer.position.x / layer.scaleX);
    int lpy = (int)(winSize.height / 2) - (int)(layer.position.y / layer.scaleY);

    int dx = lpx - position.x;
    int dy = lpy - position.y;
    
    int nx = lpx;
    int ny = lpy;
    
    if(abs(dx) > distance.width){
        if(dx < 0){
            nx = lpx + (abs(dx) - distance.width);
        }else{
            nx = lpx - (abs(dx) - distance.width);
        }
    }
    if(abs(dy) > distance.height){
        if(dy < 0){
            ny = lpy + (abs(dy) - distance.height);
        }else{
            ny = lpy - (abs(dy) - distance.height);
        }
    }
    
    CGPoint viewPoint = ccpSub(ccp(winSize.width / 2, winSize.height / 2), ccp(nx, ny));
    
    viewPoint.x = MIN(viewPoint.x, 0);
    viewPoint.y = MIN(viewPoint.y, 0);
    
    int mx = (((int)tileMap.mapSize.width * (int)tileMap.tileSize.width) - (int)winSize.width) * -1;
    int my = (((int)tileMap.mapSize.height * (int)tileMap.tileSize.height) - (int)winSize.height) * -1;
    
    viewPoint.x = MAX(viewPoint.x, mx);
    viewPoint.y = MAX(viewPoint.y, my);
    
	int x = (viewPoint.x * layer.scaleX);
	int y = (viewPoint.y * layer.scaleY);
    
    layer.position = ccp(x, y);
}

+(void)setViewpointScreen:(CCLayer*)layer tileMap:(CCTMXTiledMap*)tileMap position:(CGPoint)position{
    CGSize winSize = [[CCDirector sharedDirector] winSize];

    int vx = (((int)(position.x / winSize.width) * winSize.width) + (int)(winSize.width / 2));
    int vy = (((int)(position.y / winSize.height) * winSize.height) + (int)(winSize.height / 2));

    CGPoint viewPoint = ccpSub(ccp(winSize.width / 2, winSize.height / 2), ccp(vx, vy));
    
	int x = (viewPoint.x * layer.scaleX);
	int y = (viewPoint.y * layer.scaleY);

    layer.position = ccp(x, y);
}

+(void)setViewpointScreenScroll:(CCLayer*)layer tileMap:(CCTMXTiledMap*)tileMap position:(CGPoint)position{
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    int vx = (((int)(position.x / winSize.width) * winSize.width) + (int)(winSize.width / 2));
    int vy = (((int)(position.y / winSize.height) * winSize.height) + (int)(winSize.height / 2));
    
    CGPoint viewPoint = ccpSub(ccp(winSize.width / 2, winSize.height / 2), ccp(vx, vy));
    
	int x = (viewPoint.x * layer.scaleX);
	int y = (viewPoint.y * layer.scaleY);
    
    if(self.isAction == FALSE && (layer.position.x != x || layer.position.y != y)){
        [self setAction:TRUE];
        
        NSMutableArray* actions = [NSMutableArray array];
        [actions addObject:[CCMoveTo actionWithDuration:1.0f position:ccp(x, y)]];
        [actions addObject:[CCCallFunc actionWithTarget:self selector:@selector(setActionFalse)]];
        
        id action;
        action = [CCSequence actionsWithArray:actions];
        [layer stopAllActions];
        [layer runAction:action];    
    }
}

@end
